# Generated by Django 2.0.7 on 2018-09-11 08:27

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('document_generator', '0002_documenttemplate_uid'),
    ]

    operations = [
        migrations.AlterField(
            model_name='documenttemplate',
            name='uid',
            field=models.CharField(default=uuid.uuid4, max_length=256),
        ),
    ]
