# Generated by Django 2.0.13 on 2019-07-11 16:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tropp', '0003_auto_20190122_1551'),
    ]

    operations = [
        migrations.AlterField(
            model_name='picture',
            name='date',
            field=models.DateTimeField(null=True, verbose_name='Date'),
        ),
    ]
