# Generated by Django 2.0.9 on 2019-01-02 14:08

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('events', '0003_auto_20181120_1059'),
    ]

    operations = [
        migrations.AlterField(
            model_name='eventhandler',
            name='settings',
            field=django.contrib.postgres.fields.jsonb.JSONField(default=dict),
        ),
    ]
