# Generated by Django 2.0.7 on 2018-09-04 07:24

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('document_generator', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='documenttemplate',
            name='uid',
            field=models.CharField(default=uuid.UUID('c32ff5a4-3df1-4d1e-9319-7f47c64f4f35'), max_length=256),
        ),
    ]
