# Generated by Django 2.0.9 on 2018-11-20 10:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('datastore', '0002_auto_20181024_1609'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='datastore',
            options={'ordering': ['key']},
        ),
        migrations.AlterModelOptions(
            name='datastorepermission',
            options={'ordering': ['prefix'], 'permissions': (('can_read_datastore', "Is able to read all datastore's elements"), ('can_readwrite_datastore', 'Is able to write in datastore'))},
        ),
        migrations.AlterModelOptions(
            name='relateddocument',
            options={'ordering': ['key']},
        ),
    ]
