# Generated by Django 2.0.13 on 2019-04-17 16:30

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('accounts', '0004_auto_20181120_1059'),
    ]

    operations = [
        migrations.AlterField(
            model_name='terrauser',
            name='uuid',
            field=models.CharField(default=uuid.uuid4, max_length=255, unique=True, verbose_name='unique identifier'),
        ),
    ]
