# Generated by Django 2.0.9 on 2018-11-20 10:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('document_generator', '0004_auto_20181012_0755'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='documenttemplate',
            options={'ordering': ['id'], 'permissions': (('can_upload_template', 'Is allowed to upload a template'), ('can_update_template', 'Is allowed to update a template'), ('can_delete_template', 'Is allowed to delete a template'))},
        ),
        migrations.AlterModelOptions(
            name='downloadabledocument',
            options={'ordering': ['id']},
        ),
    ]
