# Generated by Django 2.0.13 on 2019-03-29 10:56

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('terra', '0018_auto_20190326_1049'),
        ('terra', '0018_auto_20190228_1655'),
    ]

    operations = [
    ]
