# Generated by Django 2.0.13 on 2019-05-03 15:26

from django.db import migrations, models
import terracommon.terra.models


class Migration(migrations.Migration):

    dependencies = [
        ('terra', '0022_auto_20190417_1610'),
    ]

    operations = [
        migrations.AddField(
            model_name='layer',
            name='geom_type',
            field=models.IntegerField(choices=[(terracommon.terra.models.GeometryTypes(0), 0), (terracommon.terra.models.GeometryTypes(1), 1), (terracommon.terra.models.GeometryTypes(3), 3), (terracommon.terra.models.GeometryTypes(4), 4), (terracommon.terra.models.GeometryTypes(5), 5), (terracommon.terra.models.GeometryTypes(6), 6), (terracommon.terra.models.GeometryTypes(7), 7)], null=True),
        ),
    ]
