# Generated by Django 2.1.5 on 2019-01-22 14:51

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tropp', '0002_auto_20181123_1028'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='picture',
            options={'get_latest_by': 'date', 'ordering': ['-date'], 'permissions': (('change_state_picture', 'Is able to change the picture state'),)},
        ),
    ]
