# Generated by Django 2.0.9 on 2018-10-31 09:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('events', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='eventhandler',
            name='priority',
            field=models.PositiveIntegerField(default=10),
        ),
    ]
