# Generated by Django 2.0.9 on 2018-10-23 15:22

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('accounts', '0002_auto_20180911_0818'),
    ]

    operations = [
        migrations.AlterField(
            model_name='terrauser',
            name='uuid',
            field=models.CharField(default=uuid.uuid4, editable=False, max_length=255, unique=True, verbose_name='unique identifier'),
        ),
    ]
