# Generated by Django 2.0.13 on 2019-07-25 14:00

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('tropp', '0006_picture_order_by'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='picture',
            options={'get_latest_by': 'date', 'ordering': ['-date'], 'permissions': (('change_state_picture', 'Is able to change the picture state'),)},
        ),
        migrations.AlterField(
            model_name='picture',
            name='date',
            field=models.DateTimeField(default=django.utils.timezone.now, verbose_name='Date'),
            preserve_default=False,
        ),
    ]
