# Generated by Django 2.0.7 on 2018-08-24 10:17

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('trrequests', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='userrequest',
            options={'permissions': (('can_create_requests', 'Is able to create a new requests'), ('can_read_self_requests', 'Is able to get own requests'), ('can_read_all_requests', 'Is able to get all requests'), ('can_comment_requests', 'Is able to comment an user request'), ('can_internal_comment_requests', 'Is able to add comments not visible by users'), ('can_read_comment_requests', 'Is allowed to read only non-internal comments'), ('can_change_state_requests', 'Is authorized to change the request state'), ('can_download_all_pdf', 'Is able to download a pdf document'))},
        ),
    ]
