# Generated by Django 2.0.9 on 2018-11-23 09:28

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tropp', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='viewpoint',
            name='themes',
        ),
        migrations.DeleteModel(
            name='Theme',
        ),
    ]
